import java.awt.*;
import java.awt.image.*;
import java.io.*;
import javax.imageio.*;
import javax.swing.*;
import javax.swing.text.*;

/**
 * Eigenes Grafik-Panel
 * 
 * @author Toens
 * @version v0.1
 */
public class JGrafikPanel extends JPanel {
    public int myParameter;

    private BufferedImage bildObjekt;
    private int veraenderungsart;

    public JGrafikPanel() {
        veraenderungsart = 0;
        myParameter = 0;
    }
    
    public void setVeraenderungsart(int art) {
        veraenderungsart = art;
    }

    public void setBild(String bildname) {
        try {
            bildObjekt = ImageIO.read(new File(bildname));
            BilderGUI.originalbild = JGrafikPanel.deepCopy(bildObjekt); // Originalbild retten                
            // z.B.: bildObjekt = ImageIO.read(new File("pic_erdbeere.png"));
        } catch (IOException e) {
            System.out.println("GrafikPanel Error: "+e);
            bildObjekt = null;
        }
        repaint();
    }

    /**
     * Kernstueck des LichtPanels: paintComponent ist eine Methode, die
     * eigentlich von der Oberklasse JPanel stammt (unser JLichtPanel erbt
     * ja von der JPanel-Klasse). Hier wird die Methode ueberschrieben.
     * Wir sorgen hier naemlich dafuer, dass unsere eigenen Inhalte dort
     * hineingezeichnet werden. Die Methode wird immer dann vom Fenstersystem
     * (bzw. indirekt vom Windows-Betriebssystem) aufgerufen, wenn der Inhalt
     * neu gezeichnet werden soll.
     */
    public void paintComponent(Graphics g) {
        super.paintComponent(g); // Zur Sicherheit die urspruengliche Methoder der Oberklasse aufrufen

        Rectangle r = getBounds(null);
        int h = r.height; // Höhe
        int w = r.width;  // Breite
        
        if(bildObjekt == null) {
            bildObjekt = new BufferedImage(h,w,BufferedImage.TYPE_INT_RGB);
        }
    
        veraendereBild();

        g.drawImage(bildObjekt, 0, 0, null);

        //g.setColor(Color.gray);  
        //g.fillRect(0,0,w,h);       // Hintergrund grau füllen

    }

    public static BufferedImage deepCopy(BufferedImage bi) {
        ColorModel cm = bi.getColorModel();
        boolean isAlphaPremultiplied = cm.isAlphaPremultiplied();
        WritableRaster raster = bi.copyData(bi.getRaster().createCompatibleWritableRaster());
        return new BufferedImage(cm, raster, isAlphaPremultiplied, null);
    }    
    
    public static int getFarbe(int x, int y) {
        int w = BilderGUI.originalbild.getWidth();
        int h = BilderGUI.originalbild.getHeight();
        if(x<0) {x=0;}
        if(y<0) {y=0;}
        if(x>=w) {x=(w-1);}
        if(y>=h) {y=(h-1);}
        return BilderGUI.originalbild.getRGB(x,y);
    }
    
    public void veraendereBild() {
        if(veraenderungsart == 0) {
            // nichts tun!
        }
        if(veraenderungsart == 1) {
            blauton();
        }
        if(veraenderungsart == 2) {
            verzerren();
        }
        if(veraenderungsart == 3) {
            muster();
        }
    }
    
    public void blauton() {
        BufferedImage bi = bildObjekt;
        int breite = bi.getWidth();
        int hoehe = bi.getHeight();

        for(int x = 0; x< breite; x++) {
            for(int y = 0; y<hoehe; y++) {
                // farbcode der Koordinate x,y ermitteln
                double xneu = x;
                double yneu = y;
                int farbcode = getFarbe((int)xneu , (int) yneu);

                // Farbanteile extrahieren (BESSER NICHT VERAENDERN!)
                int blau = (farbcode & 255);
                int gruen = (farbcode & (255*256))/256;
                int rot = (farbcode & (255*256*256))/(256*256);

                // neue Farbe festlegen
                int blauneu = blau;
                int gruenneu = gruen - myParameter*2;
                int rotneu = rot - myParameter*2;
                
                // neue Farbwerte müssen im Bereich 0..255 liegen: (BESSER NICHT VERAENDERN!)
                if(blauneu < 0) { blauneu = 0;}
                if(gruenneu < 0) { gruenneu = 0;}
                if(rotneu < 0) { rotneu = 0;}
                if(blauneu > 255) { blauneu = 255;}
                if(gruenneu > 255) { gruenneu = 255;}
                if(rotneu > 255) { rotneu = 255;}

                // neuen Farbcode berechnen (BESSER NICHT VERAENDERN!)
                farbcode = rotneu*256*256 + gruenneu*256 + blauneu;

                // neuen Farbcode an der angegebenen Koordinate setzen
                bi.setRGB(x,y,farbcode);
                
            }
        }
    }
    
    
    
  
    
    public void verzerren() {
        BufferedImage bi = bildObjekt;
    
        int breite = bi.getWidth();
        int hoehe = bi.getHeight();

        for(int x = 0; x< breite; x++) {
            for(int y = 0; y<hoehe; y++) {
                // farbcode der Koordinate x,y ermitteln
                double xneu = (x + myParameter * Math.sin(Math.PI*2*x/300));
                double yneu = (y + myParameter * Math.sin(Math.PI*2*y/450));
                int farbcode = getFarbe((int)xneu , (int) yneu);

                // Farbanteile extrahieren (BESSER NICHT VERAENDERN!)
                int blau = (farbcode & 255);
                int gruen = (farbcode & (255*256))/256;
                int rot = (farbcode & (255*256*256))/(256*256);

                // neue Farbe festlegen
                int blauneu = blau;
                int gruenneu = gruen;
                int rotneu = rot;

                // neue Farbwerte müssen im Bereich 0..255 liegen: (BESSER NICHT VERAENDERN!)
                if(blauneu < 0) { blauneu = 0;}
                if(gruenneu < 0) { gruenneu = 0;}
                if(rotneu < 0) { rotneu = 0;}
                if(blauneu > 255) { blauneu = 255;}
                if(gruenneu > 255) { gruenneu = 255;}
                if(rotneu > 255) { rotneu = 255;}

                // neuen Farbcode berechnen (BESSER NICHT VERAENDERN!)
                farbcode = rotneu*256*256 + gruenneu*256 + blauneu;

                // neuen Farbcode an der angegebenen Koordinate setzen
                bi.setRGB(x,y,farbcode);
            }
        }    
    }
    
    public void muster() {
        BufferedImage bi = bildObjekt;
        int breite = bi.getWidth();
        int hoehe = bi.getHeight();

        for(int x = 0; x< breite; x++) {
            for(int y = 0; y<hoehe; y++) {
                // farbcode der Koordinate x,y ermitteln
                double xneu = x;
                double yneu = y;
                int farbcode = getFarbe((int)xneu , (int) yneu);

                // Farbanteile extrahieren (BESSER NICHT VERAENDERN!)
                int blau = (farbcode & 255);
                int gruen = (farbcode & (255*256))/256;
                int rot = (farbcode & (255*256*256))/(256*256);

                // neue Farbe festlegen
                int blauneu = blau;
                int gruenneu = gruen;
                int rotneu = rot;
                
                // ggf. Farbe ändern
                if( (x >= myParameter) && (x<=myParameter+10 )) {
                    blauneu = 255;
                    gruenneu = 255;
                    rotneu = 255;
                }

                // neue Farbwerte müssen im Bereich 0..255 liegen: (BESSER NICHT VERAENDERN!)
                if(blauneu < 0) { blauneu = 0;}
                if(gruenneu < 0) { gruenneu = 0;}
                if(rotneu < 0) { rotneu = 0;}
                if(blauneu > 255) { blauneu = 255;}
                if(gruenneu > 255) { gruenneu = 255;}
                if(rotneu > 255) { rotneu = 255;}

                // neuen Farbcode berechnen (BESSER NICHT VERAENDERN!)
                farbcode = rotneu*256*256 + gruenneu*256 + blauneu;

                // neuen Farbcode an der angegebenen Koordinate setzen
                bi.setRGB(x,y,farbcode);
            }
        }
    }
    
    

    
}
